@@ -12,34 +12,26 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from neutron_lib.api.definitions import provider_net as pnet
-from neutron_lib.api import validators
-from neutron_lib.db import model_base
-from neutron_lib import exceptions as n_exc
-
-from midonet.neutron._i18n import _, _LW
 from midonet.neutron.common import constants as m_const
+from neutron.api.v2 import attributes
+from neutron.db import model_base
 from neutron.db import models_v2
-from neutron.plugins.common import constants as p_const
+from neutron.extensions import providernet as pnet
+from neutron import i18n
 from oslo_log import log as logging
 import sqlalchemy as sa
 from sqlalchemy import orm
 
 
 LOG = logging.getLogger(__name__)
-
-
-_MIDONET_TYPES = [
-    p_const.TYPE_LOCAL,
-    m_const.TYPE_MIDONET,
-]
+_LW = i18n._LW
 
 
 class NetworkBinding(model_base.BASEV2):
     __tablename__ = 'midonet_network_bindings'
 
-    network_id = sa.Column(sa.String(36), sa.ForeignKey('networks.id'),
-                           primary_key=True)
+    network_id = sa.Column(sa.String(36), sa.ForeignKey('networks.id',
+                           ondelete="CASCADE"), primary_key=True)
     network_type = sa.Column(sa.String(length=255), nullable=False)
     network = orm.relationship(models_v2.Network,
                                backref=orm.backref("network_binding",
@@ -55,34 +47,30 @@ class MidonetProviderNetworkMixin(object):
             network_id=network_id).first()
         if net_binding:
             return net_binding.network_type
-        return m_const.TYPE_MIDONET
+        return None
 
     def _extend_provider_network_dict(self, context, network):
         id = network['id']
-        network[pnet.NETWORK_TYPE] = self._get_net_type(context.session, id)
+        net_type = self._get_net_type(context.session, id)
+        if net_type:
+            network[pnet.NETWORK_TYPE] = net_type
 
     def _process_provider_create(self, network):
 
         net_type = network.get(pnet.NETWORK_TYPE)
-        if not validators.is_attr_set(net_type):
-            return None
-
-        if net_type in _MIDONET_TYPES:
-            # NOTE(yamamoto): Accept a few well-known types as
-            # the default type.  This is a workaround for Horizon, which
-            # currently doesn't have a way to specify MidoNet network types
-            # or "no provider network".
-            # REVISIT(yamamoto): Clean this up once Horizon is fixed
-            if net_type != m_const.TYPE_MIDONET:
-                LOG.warning(_LW('Unsupported network type %(type)s detected '
-                                'in a create network request.'),
-                            {'type': net_type})
+        if not attributes.is_attr_set(net_type):
             return None
 
+        # Instead of validating unsupported inputs, just log and ignore them.
+        # Horizon, for example, forces you to select a type, and because
+        # 'uplink' type is not known to Horizon, requests from Horizon would
+        # always include an unsupported type would be sent for external
+        # networks. To avoid completely stopping the user from proceeding in
+        # that case, log and continue on.
         if net_type != m_const.TYPE_UPLINK:
-            msg = _('Unsupported network type %(type)s detected '
-                    'in a create network request.') % {'type': net_type}
-            raise n_exc.InvalidInput(error_message=msg)
+            LOG.warning(_LW('Unsupported network type %(type)s detected in a '
+                            'create network request.'), {'type': net_type})
+            net_type = None
 
         return net_type
 
@@ -101,7 +89,7 @@ class MidonetProviderNetworkMixin(object):
         if not filters:
             return True
 
-        if validators.is_attr_set(network.get(pnet.NETWORK_TYPE)):
+        if attributes.is_attr_set(network.get(pnet.NETWORK_TYPE)):
             return self._match_attrs(network, filters)
         else:
             return True
