@@ -21,8 +21,10 @@ from neutron.common import constants as n_const
 from neutron.common import exceptions as n_exc
 from neutron.db import allowedaddresspairs_db as addr_pair_db
 from neutron.db import extraroute_db
+from neutron.db import portsecurity_db as ps_db
 from neutron.extensions import allowedaddresspairs as addr_pair
 from neutron.extensions import extra_dhcp_opt as edo_ext
+from neutron.extensions import portsecurity as psec
 from neutron.extensions import providernet as pnet
 from neutron.extensions import securitygroup as ext_sg
 from neutron import i18n
@@ -42,7 +44,8 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
                       am_db.AgentMembershipDbMixin,
                       extraroute_db.ExtraRoute_db_mixin,
                       pnet_db.MidonetProviderNetworkMixin,
-                      pb_db.MidonetPortBindingMixin):
+                      pb_db.MidonetPortBindingMixin,
+                      ps_db.PortSecurityDbMixin):
 
     supported_extension_aliases = [
         'agent',
@@ -53,6 +56,7 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
         'external-net',
         'extra_dhcp_opt',
         'extraroute',
+        'port-security',
         'provider',
         'quotas',
         'router',
@@ -79,6 +83,9 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
             self._process_l3_create(context, net, net_data)
             self._create_provider_network(context, net_data)
             self._extend_provider_network_dict(context, net)
+            if psec.PORTSECURITY in net_data:
+                self._process_network_port_security_create(context, net_data,
+                                                           net)
             self.client.create_network_precommit(context, net)
 
         try:
@@ -137,6 +144,10 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
                 context, id, network)
             self._process_l3_update(context, net, network['network'])
             self._extend_provider_network_dict(context, net)
+            if psec.PORTSECURITY in network['network']:
+                self._process_network_port_security_update(context,
+                                                           network['network'],
+                                                           net)
             self.client.update_network_precommit(context, id, net)
 
         self.client.update_network_postcommit(id, net)
@@ -238,8 +249,22 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
             # Update fields
             port_data.update(new_port)
 
-            # Bind security groups to the port
-            self._ensure_default_security_group_on_port(context, port)
+            port_psec, has_ip = self._determine_port_security_and_has_ip(
+                context, port['port'])
+            port['port'][psec.PORTSECURITY] = port_psec
+            self._process_port_port_security_create(context,
+                                                    port['port'],
+                                                    new_port)
+
+            if port_psec is False:
+                if self._check_update_has_security_groups(port):
+                    raise psec.PortSecurityAndIPRequiredForSecurityGroups()
+                if self._check_update_has_allowed_address_pairs(port):
+                    raise addr_pair.AddressPairAndPortSecurityRequired()
+            else:
+                # Bind security groups to the port
+                self._ensure_default_security_group_on_port(context, port)
+
             sg_ids = self._get_security_groups_on_port(context, port)
             self._process_port_create_security_group(context, new_port, sg_ids)
 
@@ -323,6 +348,15 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
             self.update_address_pairs_on_port(context, id, port,
                                               original_port, p)
 
+            self._process_port_port_security_update(context, port['port'], p)
+
+            port_psec = p.get(psec.PORTSECURITY)
+            if port_psec is False:
+                if p.get(ext_sg.SECURITYGROUPS):
+                    raise psec.PortSecurityPortHasSecurityGroup()
+                if p.get(addr_pair.ADDRESS_PAIRS):
+                    raise addr_pair.AddressPairAndPortSecurityRequired()
+
             self.client.update_port_precommit(context, id, p)
 
         self.client.update_port_postcommit(id, p)
@@ -670,3 +704,51 @@ class MidonetPluginV2(plugin.MidonetMixinBase,
         if not agent:
             agent = super(MidonetPluginV2, self).get_agent(context, id, fields)
         return agent
+
+    # Patch PortSecurityDbCommon to fix up the lack of DB rows at runtime.
+    # See bug/1507651 for details.
+    # REVISIT(yamamoto): The most of these should be fixed in the base class
+
+    def _extend_port_security_dict(self, response_data, db_data):
+        # NOTE(yamamoto): This method is used for both of networks and ports
+        if db_data.get('port_security') is None:
+            response_data[psec.PORTSECURITY] = (
+                psec.EXTENDED_ATTRIBUTES_2_0['networks']
+                [psec.PORTSECURITY]['default'])
+        else:
+            response_data[psec.PORTSECURITY] = (
+                                db_data['port_security'][psec.PORTSECURITY])
+
+    def _get_network_security_binding(self, context, network_id):
+        # NOTE(yamamoto): used via _determine_port_security_and_has_ip
+        try:
+            return super(MidonetPluginV2,
+                         self)._get_network_security_binding(context,
+                                                             network_id)
+        except psec.PortSecurityBindingNotFound:
+            return (psec.EXTENDED_ATTRIBUTES_2_0['networks']
+                    [psec.PORTSECURITY]['default'])
+
+    def _get_port_security_binding(self, context, network_id):
+        # NOTE(yamamoto): Currently not used by our plugin.
+        # Override just to detect future problems earlier.
+        raise NotImplementedError()
+
+    def _process_network_port_security_update(self, context, network_req,
+                                              network_res):
+        try:
+            super(MidonetPluginV2, self)._process_network_port_security_update(
+                context, network_req, network_res)
+        except psec.PortSecurityBindingNotFound:
+            self._process_network_port_security_create(context, network_req,
+                                                       network_res)
+
+    def _process_port_port_security_update(self, context, port_req, port_res):
+        try:
+            super(MidonetPluginV2, self)._process_port_port_security_update(
+                context, port_req, port_res)
+        except psec.PortSecurityBindingNotFound:
+            self._process_port_port_security_create(context, port_req,
+                                                    port_res)
+
+    # End of patching
